;; with apologies to gahan wilson, the original author of the science fiction
;; story writer flow chart

(setq *story-line-length* 60)

(defun ts ()
  ;; type out a story
  (print-a-story (tellstory)))

(defun tellstory () (write-a-story 1))

(defun write-a-story (statename)
   (if statename                                       ; quit if null
      (let (arc next 
            (state (cdr (assoc statename *the-story-list*))))
         ;; find the state in the network and select an arc from it. 
         (when state
            (setq arc (random-element state))
            ;; separate the words to be printed and the next-state-name. 
            (setq next (car arc) arc (cdr arc))
            ;; if there is more than one next-state-names,
	    ;; randomly select one. 
            (or (atom next) (setq next (random-element next)))
            ;; recurse. 
            (cons-or-append (expand-a-list arc) (write-a-story next))))))

(defun expand-a-list (list)
       ;; recursively expands word lists - could contain (:@ :foo)'s
       (cond ((atom list) list)
	     ((and (consp (car list)) (eq (caar list) :@))
	      (cons-or-append
	       (expand-a-list
		(random-element (cdr (assoc (cadar list) *expand-list*))))
	       (expand-a-list (cdr list))))
	     (t (cons-or-append (car list)
				(expand-a-list (cdr list))))))

(defun random-element (l)
  ;; returns a random element from the list l.
  (nth (random (length l)) l))

(defun cons-or-append (x y)
       ;; does either a cons or an append.
       (cond ((null x) y)
	     ((atom x) (cons x y))
	     ((append x y))))

(defun print-a-story (l)
  (do* ((capitalize t)
	(length-left *story-line-length*
		     (if (zerop length-left)
		       (progn (terpri) *story-line-length*)
		       length-left))
	(pword nil)
	(wlen nil)
	(wordsleft l (cdr wordsleft))
	period-seen
	)
       ((null wordsleft) "")

     (setf pword (string-downcase (symbol-name (car wordsleft)))
	   wlen (length pword))
     (if (eq (cadr wordsleft) :period)
       (setf pword (strcat pword ".")
	     period-seen t
	     wordsleft (cdr wordsleft)
	     wlen (1+ wlen))
       (setf period-seen nil))
     (if (eq (cadr wordsleft) :comma)
       (setf pword (strcat pword ",")
	     wordsleft (cdr wordsleft)
	     wlen (1+ wlen)))
     (if capitalize
       (setf capitalize nil
	     pword (capitalize pword)))
     (when (> wlen length-left)
       (terpri)
       (setf length-left *story-line-length*))
     (princ pword)
     (setf length-left (- length-left wlen))
     (when period-seen
       (if (and (> length-left 0) (not (= length-left *story-line-length*)))
	 (princ " ")
	 (progn (terpri) (setf length-left *story-line-length*)))
       (setf period-seen nil
	     capitalize t))
     (if (> length-left 0)
       (if (not (= length-left *story-line-length*))
	 (progn (princ " ") (setf length-left (1- length-left))))
       (progn (terpri) (setf length-left *story-line-length*)))
     )
  (terpri))

(defun capitalize (str)
  (strcat (string-upcase (subseq str 0 1)) (subseq str 1)))

;; the script

(setq *the-story-list*
   '((1 (4 (:@ :when) a (:@ :adj1) (:@ :adj2) (:@ :object) (:@ :lands-verb)
	   (:@ :where) :period (:@ :finder))
	(2 the earth)
	(5 the earth is (:@ :attacked-verb) by) 
	(4 (:@ :when) (:@ :striking-object) (:@ :strike-verb) the earth :period
	   (:@ :finder))
	(4 (:@ :when) (:@ :where) (:@ :finder)))
     (2 (3 burns up and)
	(3 freezes and)
	(3 falls into the sun and) 
	(5 is (:@ :attacked-verb) by)
	(5 is (:@ :attacked-verb) by)
	(5 is (:@ :attacked-verb) by)
	(5 is (:@ :attacked-verb) by)
	((3 6) is struck by (:@ :striking-object) and))
      (3 (6)
	 (999 everybody dies :period)
	 (1 almost everybody dies :comma but then) 
         (999 almost everybody dies :period))
      (4 (7 (:@ :find-verb)))
      (5 (13 (:@ :number) (:@ :size) (:@ :source-of-monsters) (:@ :monsters))) 
      (6 (999 is destroyed :period)
	 (999 is saved :period)
	 (3 is not destroyed but)) 
      (7 (12 (:@ :size)))
      (12 (13 (:@ :monsters)))
      (13 (14 which)) 
      (14 ((15 16) want our women)
	  (999 are friendly :period) 
	  (15 are friendly but misunderstood)
	  (15 misunderstand us) 
	  (15 understand us all too well) 
	  ((15 17) view us as (:@ :people-quality)) 
	  ((15 17) consider us to be (:@ :people-quality)) 
	  ((15 17) look upon us only as (:@ :people-quality)))
      (15 (18 :period a wise old scientist (:@ :from-school) predicts that they)
	  (18 :period it turns out that they)
	  (18 :period it is discovered that they)
	  (18 :period scientists (:@ :from-school) (:@ :guess) that they)
	  (18 :period the military fears that they))
      (16 (999 take a few and leave :period))
      (17 (999 and eat us :period)) 
      (18 (20 (:@ :quality)))
      (20 (18 and) (21 :period the monsters) (21 :period they)) 
      (21 (22 can be (:@ :killed-verb) by (:@ :killers)) 
	  ((24 25 25) cannot be (:@ :killed-verb) by (:@ :killers)))
      (22 (999 :period)
	  (999 :period)
	  (999 and are :period)
	  (1 but then :comma) 
	  (999 so everyone gets married and the sun sets :period) 
	  (999 and thus the world was saved :period))
      (24 (999 but they die from catching chicken pox :period)
	  (3 and) 
	  (999 so they kill us :period) 
	  (999 so they put us under a benign dictatorship :period) 
	  (999 so they eat us :period))
      (25 (26 so two bright young scientists
	      (:@ :from-school) (:@ :make-verb) (:@ :weapon))
	  (26 so scientists (:@ :from-school) (:@ :make-verb) (:@ :weapon))
	  (27 but))
      (26 (24 which fails)
	  (24 which only makes them (:@ :angry)) 
	  (27 which only makes them (:@ :angry) :comma but)
	  (27 which fails :comma but) 
	  (999 which kills them :period)
	  (1 which kills them :comma but then) 
	  (999 which turns them into disgusting lumps :period))
      (27 (28 a cute little kid convinces them people are ok) 
	  (28 a priest talks to them of god) 
	  (28 they fall in love with this beautiful girl) 
	  (29 they fall in love with this beautiful girl))
      (28 (999 and they die :period)
	  (999 and they leave :period) 
	  (999 and they turn into disgusting lumps :period)
	  (999 and they stay on earth :comma intermarry with dolphins
	       :comma and create a superrace :period))
      (29 (999 and they get married and live happily forever after :period))
      (999 (nil the end :period))))

(setq *expand-list*
  '((:people-quality
     (a source of nourishment)
     (an inferior race)
     (tasty between meal treats)
     (mortal weaklings)
     (an embarrassment to the universe)
     (a disease)
     (inferiors)
     (potential subjects to be dominated)
     (pets))
    (:monsters 
     worms zombies insects ants lizards bugs reptiles (icky things) 
     (carnivorous plants) ghouls amoebae demons)
    (:find-verb (is attacked by)
		(is the first to see)
		(accidentally releases)
		discovers
		uncovers
		(digs up)
		finds
		(has a premonition of))
    (:finder ((:@ :old-adj) hobo)
	     (a group of (:@ :group))
	     (a group of (:@ :group))
	     ((:@ :old-adj) prospector)
	     ((:@ :old-adj) wino)
	     (a simple night watchman))
    (:make-verb invent create make)
    (:size nil nil towering tiny enormous giant microscopic)
    (:quality
     (are radioactive)
     (like to eat cities)
     (are only the first wave of an invasion)
     (will ruin the neighborhood)
     (are developing a taste for human (:@ :people-stuff))
     (can read our minds)
     (have x-ray vision)
     (are four dimensional)
     (are hungry)
     (are afraid of fire)
     (are invisible at times)
     (are seemingly invincible)
     (leave trails of slime behind them)
     (must eat human (:@ :people-stuff) to live)
     (feed only on living (:@ :people-stuff))
     (fly at supersonic speeds)
     (are capable of supersonic flight)
     (have insatiable libidos))
    (:killers
     (a crowd of peasants with torches)
     (the fourth tank batallion)
     (the (:@ :branch))
     (a direct hit with a hydrogen bomb)
     (the common cold)
     (the armed forces)
     (the united armed forces of the world)
     (the japanese air force)
     (a mob of peasants with pitch forks)
     (the atomic bomb))
    (:branch army navy marines (coast guard) mafia)
    (:group scientists (simple farmers) (hard-boiled newpaper reporters)
	    farmers children (nuclear physicists) (evil scientists)
	    (crazy kids) scientists miners teenagers fishermen
	    geishas (high school dropouts) missionaries
	    (intelligent marsupials) gypsies)
    (:attacked-verb attacked ravaged (set upon) (suddenly attacked) deluged)
    (:when (just after sunset)
	   (early one morning :comma)
	   (late one night :comma)
	   nil
	   nil
	   nil
	   (on the first day of spring)
	   (one halloween))
    (:where (in the nevada desert)
	    (near the arctic circle)
	    (in the jungles of the amazon)
	    (on the campus of a small liberal arts college)
	    (in a small english village)
	    (in the everglades)
	    nil
	    nil
	    nil
	    nil
	    (down the street from your house)
	    (on the north shore of lake superior)
	    (in a bar)
	    (on the summit of an ancient aztec pyramid)
	    (at a secure military base deep in antarctica))
    (:nationality nil english nil japanese american nil)
    (:weapon (a new weapon)
	     (a death device)
	     (a kind of death ray)
	     ( a powerful  (nuclear atomic) gun)
	     (a giant robot))
    (:killed-verb killed destroyed beaten)
    (:from-school
     (from the coordinated science laboratory of the university of illinois)
     nil
     nil
     nil
     nil
     nil
     (from m\.i\.t\.)
     (from stanford)
     (from the university of illinois)
     (from the university of illinois)
     macalester)
    (:striking-object (a giant comet)
		      (a large meteor)
		      (a strange flashing object)
		      (a chunk of unknown metal from an alien spacecraft)
		      (a malfunctioning satellite))
    (:strike-verb strikes hits (collides with))
    (:landing-object (:@ :adj1) (:@ :adj2) (:@ :object))
    (:adj1 strange large)
    (:adj2 glowing pulsating flashing)
    (:object object thing ball sphere)
    (:lands-verb lands (sets down) crashes)
    (:angry angry mad angrier madder hungry wince (feel good)
	    (want to laugh))
    (:old-adj (an old) (a run down))
    (:guess predict believe fear (have a theory) hypothesize)
    (:number nil
	     nil
	     nil
	     two
	     three
	     (a swarm of)
	     (hordes of)
	     (thousands of)
	     (millions of)
	     (billions of)
	     (a few)
	     countless)
    (:source-of-monsters prehistoric
			 moon
			 martian
			 betelgeusian
			 extragalactic)
    (:people-stuff blood flesh brains)))

;; message to the user:

(format t "Type \"(ts)\" to generate a story.~%")
